
/**
 * An immutable 2D vector
 *
 * @author Michael Yu
 */
public class Vect2D {

    /**
     * Construct a new vector with coordinates (0,0)
     */
    public Vect2D() {
        this(0,0);
    }

    /**
     * Construct a new vector with coordinates (x,y)
     *
     * @param x is the x-component of the vector
     * @param y is the x-component of the vector
     */
    public Vect2D(double x, double y) {
        this.x = x;
        this.y = y;
    }

    /**
     * @return the x-component of the vector
     */
    public double getX() {
        return x;
    }

    /**
     * @return the y-component of the vector
     */
    public double getY() {
        return y;
    }

    /**
     * Calculate the dot product of 'this' vector and v
     *
     * @param v is a vector to be dotted with 'this' vector
     * @return the dot product of 'this' vector and v
     */
    public double dot(Vect2D v) {
        return x * v.getX() + y * v.getY();
    }

    /**
     * Return the sum of 'this' vector and v
     *
     * @param v is a vector to be added to 'this' vector
     * @return a new vector that is the sum of 'this' and v
     */
    public Vect2D add(Vect2D v) {
        return new Vect2D(x + v.getX(), y + v.getY());
    }

    /**
     * Scale 'this' vector by c
     *
     * @param c is the number 'this' vector will be scaled by
     * @return a new vector that is 'this' vector scaled by c
     */
    public Vect2D scale(double c) {
        return new Vect2D(x * c, y * c);
    }

    /**
     * Convert this vector to a string
     *
     * @return the vector in string form
     */
    public String toString() {
        return "(" + x + ", " + y + ")";        
    }

    private double x; // The x-component of the vector
    private double y; // The y-component of the vector
}
